package src

import (
	"acme-mana/src/crypto"
	"bytes"
	"encoding/base64"
	"encoding/hex"
	"encoding/json"
	"github.com/gin-gonic/gin"
	"log"
	"os"
	"path"
	"path/filepath"
	"strconv"
)

func InitHttpServer(host string, port int) {

	log.Println("Start http server, Listen " + strconv.Itoa(port))
	h := gin.Default()
	h.GET("/api/v1/refresh", refreshCert)
	h.GET("/api/v1/cert", getCert)
	h.GET("/api/v1/domain/list", domainList)
	err := h.Run(host + ":" + strconv.Itoa(port))
	if err != nil {
		return
	}
}

func domainList(c *gin.Context) {
	token := getToken(c)
	domains := GetAppConfig().Domains
	data, err := json.Marshal(domains)
	if err != nil {
		log.Fatal(err)
	}

	c.JSON(200, gin.H{
		"code": 200,
		"msg":  "Success",
		"data": encryptResult(string(data), token),
	})
}

func getCert(c *gin.Context) {
	name := c.Param("name")
	token := getToken(c)

	dir := GetAppConfig().CertDir
	dir = filepath.Join(dir, name)
	_, err := os.Stat(dir)
	if os.IsNotExist(err) {
		c.JSON(200, gin.H{
			"code": 500,
			"msg":  "Name does not exist.",
		})
		return
	}
	crtFilePath := path.Join(dir, CertFileName)
	crtContent, err := os.ReadFile(crtFilePath)
	if err != nil {
		c.JSON(200, gin.H{
			"code": 500,
			"msg":  "Failed to read crt file.",
		})
		return
	}
	crt := string(crtContent)

	keyFilePath := path.Join(dir, KeyFileName)
	keyContent, err := os.ReadFile(keyFilePath)
	if err != nil {
		c.JSON(200, gin.H{
			"code": 500,
			"msg":  "Failed to read key file.",
		})
		return
	}
	key := string(keyContent)

	c.JSON(200, gin.H{
		"code": 200,
		"msg":  "Success",
		"data": gin.H{
			"crt": encryptResult(crt, token),
			"key": encryptResult(key, token),
		},
	})
}

func refreshCert(c *gin.Context) {
	name := c.Param("name")
	domain := GetAppConfig().FindDomain(name)
	if domain == nil {
		c.JSON(200, gin.H{
			"code": 500,
			"msg":  "Name does not exist.",
		})
		return
	}
	Apply(*domain)
	c.JSON(200, gin.H{
		"code": 200,
		"msg":  "Success",
	})
}

func getToken(c *gin.Context) (token string) {
	token = c.Query("token")
	token = decryptParam(token)
	return
}

func decryptParam(param string) string {
	priKey := GetAppConfig().Encrypt.PriKey

	tokenBytes, err := hex.DecodeString(param)
	//tokenBytes, err := base64.StdEncoding.DecodeString(param)
	//if err != nil {
	//	log.Fatalln(err)
	//}
	tokenPlain, err := crypto.DecryptRSABase64(priKey, tokenBytes)
	if err != nil {
		log.Fatal(err)
	}
	return string(tokenPlain)

	// 使用RSA解密
	//block, _ := pem.Decode([]byte(priKey))
	//if block == nil {
	//	log.Fatal("failed to parse PEM block containing the private key")
	//}
	//privateKey, err := x509.ParsePKCS1PrivateKey(block.Bytes)
	//if err != nil {
	//	log.Fatal(err)
	//}
	//paramData := []byte(param)
	//plaintext, err := rsa.DecryptPKCS1v15(nil, privateKey, paramData)
	//if err != nil {
	//	log.Fatal(err)
	//}
	//return string(plaintext)
}

func encryptResult(content string, token string) string {
	result := crypto.EncryptAES([]byte(token), []byte(content))
	return base64.StdEncoding.EncodeToString(result)

	//key := []byte(token)
	//plaintext := []byte(content)
	//
	//block, err := des.NewCipher(key)
	//if err != nil {
	//	log.Fatal(err)
	//}
	//
	//plaintext = pad(plaintext, block.BlockSize())
	//
	//iv := make([]byte, block.BlockSize())
	//if _, err := io.ReadFull(rand.Reader, iv); err != nil {
	//	log.Fatal("Error generating random IV:", err)
	//}
	//mode := cipher.NewCBCEncrypter(block, iv)
	//
	//ciphertext := make([]byte, len(plaintext))
	//mode.CryptBlocks(ciphertext, plaintext)
	//
	//return base64.StdEncoding.EncodeToString(ciphertext)
}

func pad(src []byte, blockSize int) []byte {
	padding := blockSize - len(src)%blockSize
	padText := bytes.Repeat([]byte{byte(padding)}, padding)
	return append(src, padText...)
}
